from djaws.exceptions import AWSError


class CFError(AWSError):
    """All Cloud Formation Exceptions in the AWS App inherit from this class"""


class CFClientError(CFError):
    """All Cloud Formation Exceptions in the AWS App inherit from this class"""



import json
from importlib.resources import open_text
from unittest import mock

from botocore.exceptions import ClientError
from django.test import TestCase

from djaws import s3_api, cloudformation_api
from djaws.cloudformation.exceptions import CFError


class DescribeStacksTests(TestCase):

    @classmethod
    def setUpTestData(cls):
        with open_text('djaws.tests.schemas.cloudformation', 'describe_stacks.json') as cf_json:
            cls.r_stacks_data = json.loads(cf_json.read())

    @mock.patch.object(cloudformation_api, 'cf_session_client')
    def test_ok(self, mock_client):
        mock_client.describe_stacks.return_value = self.r_stacks_data

        r_stacks = cloudformation_api.describe_stacks(
            mock_client,
            stack_name='test123',
        )
        self.assertTrue(r_stacks.first_stack.stack_status, 'CREATE_COMPLETE')

    @mock.patch.object(cloudformation_api, 'cf_session_client')
    def test_validation_error(self, mock_client):
        mock_client.describe_stacks.side_effect = ClientError(
            {
                'Error': {
                    'Code': 'ValidationError',
                    'Message': 'Details/context around the exception or error'
                },
                'ResponseMetadata': {
                    'RequestId': '1234567890ABCDEF',
                    'HostId': 'host ID data will appear here as a hash',
                    'HTTPStatusCode': 400,
                    'HTTPHeaders': {'header metadata key/values will appear here'},
                    'RetryAttempts': 0
                }
            },
            'generate_presigned_post'
        )

        with self.assertRaises(CFError):
            cloudformation_api.describe_stacks(
                mock_client,
                stack_name='test123',
            )



from djaws.exceptions import AWSError


class CognitoError(AWSError):
    """All Cognito Exceptions in the AWS App inherit from this class"""


class CognitoClientError(CognitoError):
    """All Cognito Exceptions in the AWS App inherit from this class"""


class CognitoUserNotFoundError(CognitoClientError):
    """Cognito User not found"""


class CognitoUserNotAuthorizedError(CognitoClientError):
    """Cognito User not authorized"""


class CognitoUserNotConfirmedError(CognitoClientError):
    """Cognito User not confirmed """


class CognitoUserPasswordValidationError(CognitoClientError):
    """Cognito User password failed validation """


class CognitoUsernameExistsError(CognitoClientError):
    """Cognito User username already exists """



from django.core.management.base import BaseCommand

from djaws import cloudformation_api


class Command(BaseCommand):
    help = 'Describe Cloud Formation Stack'

    def add_arguments(self, parser):
        parser.add_argument(
            "--stack_name",
            type=str,
            help="S3 Region",
        )

    def handle(self, *args, **kwargs):

        stack_name = kwargs['stack_name']
        cf_client = cloudformation_api.cf_session_client()
        try:
            r_stacks = cloudformation_api.describe_stacks(cf_client, stack_name=stack_name)
        except Exception as e:
            self.stdout.write(self.style.ERROR(f'Error while running command:\n{str(e)}'))
            raise e
        else:
            stack = r_stacks.first_stack
            self.stdout.write(self.style.MIGRATE_LABEL(f'Successfully got stack details.'))
            msg = f'id: {stack.stack_id}\nname: {stack.stack_name}\nstatus: {stack.stack_status}'
            self.stdout.write(self.style.MIGRATE_LABEL(msg))





import json
from importlib.resources import open_text
from unittest import mock

from botocore.exceptions import ClientError
from django.test import TestCase

from djaws import s3_api
from djaws.s3.exceptions import S3Error, BucketAlreadyExists, BucketAlreadyOwnedByYouError


class CreatePublicAccessBucketTests(TestCase):
    @mock.patch.object(s3_api, 'create_bucket')
    @mock.patch.object(s3_api, 'put_public_access_block')
    @mock.patch.object(s3_api, 'put_bucket_cors')
    def test_ok(self, mock_create_bucket, mock_put_public_access_block, mock_put_bucket_cors):
        mock_create_bucket.return_value = None
        mock_put_public_access_block.return_value = True
        mock_put_bucket_cors.return_value = True

        r_bucket = s3_api.create_public_access_bucket(
            s3_api.s3_session_client(),
            bucket='my-test-bucket-2',
            region='us-west-2'
        )
        self.assertTrue(r_bucket)


class CreateBucketTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        with open_text('djaws.tests.schemas.s3', 'create_bucket.json') as bucket_json:
            cls.r_bucket_data = json.loads(bucket_json.read())

    @mock.patch.object(s3_api, 's3_session_client')
    def test_ok(self, mock_client):
        mock_client.create_bucket.return_value = self.r_bucket_data

        r_bucket = s3_api.create_bucket(
            mock_client,
            bucket='bucket1',
            region='us-west-2',
        )
        self.assertEquals(
            r_bucket.location,
            'http://examplebucket.s3.amazonaws.com/'
        )

    @mock.patch.object(s3_api, 's3_session_client')
    def test_ok_no_region(self, mock_client):
        mock_client.create_bucket.return_value = self.r_bucket_data

        r_bucket = s3_api.create_bucket(
            mock_client,
            bucket='bucket1',
            region='us-west-2',
        )
        self.assertEquals(
            r_bucket.location,
            'http://examplebucket.s3.amazonaws.com/'
        )

    @mock.patch.object(s3_api, 's3_session_client')
    def test_BucketAlreadyExists(self, mock_client):
        mock_client.create_bucket.side_effect = ClientError(
            {
                'Error': {
                    'Code': 'BucketAlreadyExists',
                    'Message': 'Details/context around the exception or error'
                },
                'ResponseMetadata': {
                    'RequestId': '1234567890ABCDEF',
                    'HostId': 'host ID data will appear here as a hash',
                    'HTTPStatusCode': 400,
                    'HTTPHeaders': {'header metadata key/values will appear here'},
                    'RetryAttempts': 0
                }
            },
            'generate_presigned_post'
        )

        with self.assertRaises(BucketAlreadyExists):
            s3_api.create_bucket(
                mock_client,
                bucket='bucket1',
                region='us-west-2',
            )


class PutPublicAccessBlockTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.access_block_data = {
            'bucket': 'bucket1',
            'config': s3_api.PublicAccessBlockConfiguration(
                block_public_acls=False,
                ignore_public_acls=False,
                block_public_policy=False,
                restrict_public_buckets=False,
            )
        }

    @mock.patch.object(s3_api, 's3_session_client')
    def test_put_public_access_block_ok(self, mock_client):
        mock_client.put_public_access_block.return_value = True

        r_access_block = s3_api.put_public_access_block(
            mock_client,
            **self.access_block_data,
        )
        self.assertTrue(r_access_block)

    @mock.patch.object(s3_api, 's3_session_client')
    def test_put_public_access_block_error(self, mock_client):
        mock_client.put_public_access_block.side_effect = ClientError(
            error_response=dict(),
            operation_name='put_public_access_block'
        )

        with self.assertRaises(S3Error):
            s3_api.put_public_access_block(
                mock_client,
                **self.access_block_data,
            )


class CreateBucketCORSTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.cors_data = {
            'bucket': 'bucket1',
            'cors_config': {
                'CORSRules': [
                    {
                        "AllowedHeaders": [
                            "*"
                        ],
                        "AllowedMethods": [
                            "GET"
                        ],
                        "AllowedOrigins": [
                            "*"
                        ],
                        "MaxAgeSeconds": 3000
                    },
                ],
            },
        }

    @mock.patch.object(s3_api, 's3_session_client')
    def test_put_bucket_cors_ok(self, mock_client):
        mock_client.put_bucket_cors.return_value = True

        r_cors = s3_api.put_bucket_cors(
            mock_client,
            **self.cors_data,
        )
        self.assertTrue(r_cors)

    @mock.patch.object(s3_api, 's3_session_client')
    def test_put_bucket_cors_error(self, mock_client):
        mock_client.put_bucket_cors.side_effect = ClientError(
            error_response=dict(),
            operation_name='put_bucket_cors'
        )

        with self.assertRaises(S3Error):
            s3_api.put_bucket_cors(
                mock_client,
                **self.cors_data,
            )


class UploadObjectTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.upload_data = {
            'data': b'test_image',
            'bucket': 'bucket1',
            'uri': 'https://example.com/test.jpg',
            'key': '998adad2-c217-402e-b259-ace44a26dff4.jpg'
        }

    @mock.patch.object(s3_api, 's3_session_client')
    def test_ok(self, mock_client):
        mock_client.put_object.return_value = None

        r_upload = s3_api.upload_object(
            mock_client,
            **self.upload_data,
        )
        self.assertEquals(
            r_upload.public_s3_url,
            'https://bucket1.s3.amazonaws.com/998adad2-c217-402e-b259-ace44a26dff4.jpg'
        )

    @mock.patch.object(s3_api, 's3_session_client')
    def test_ok_no_object_name(self, mock_client):
        mock_client.put_object.return_value = None

        upload_data = self.upload_data
        upload_data.pop('key')

        r_upload = s3_api.upload_object(
            mock_client,
            **upload_data,
        )
        self.assertTrue(r_upload.public_s3_url.startswith('https://bucket1.s3.amazonaws.com/'))
        self.assertTrue(r_upload.public_s3_url.endswith('.jpg'))

    @mock.patch.object(s3_api, 's3_session_client')
    def test_client_error(self, mock_client):
        mock_client.put_object.side_effect = ClientError(
            error_response=dict(),
            operation_name='put_object'
        )

        with self.assertRaises(S3Error):
            s3_api.upload_object(
                mock_client,
                **self.upload_data,
            )


class GeneratePresignedPostTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.generate_presigned_post = {
            'bucket': 'bucket1',
            'object_name': '998adad2-c217-402e-b259-ace44a26dff4.jpg'
        }
        with open_text('djaws.tests.schemas.s3', 'generate_presigned_post.json') as generate_presigned_post_json:
            cls.r_generate_presigned_post_data = json.loads(generate_presigned_post_json.read())

    @mock.patch.object(s3_api, 's3_session_client')
    def test_generate_presigned_post_ok(self, mock_client):
        mock_client.generate_presigned_post.return_value = self.r_generate_presigned_post_data

        r_generate_presigned_post = s3_api.generate_presigned_post(
            mock_client,
            **self.generate_presigned_post,
        )
        self.assertEquals(r_generate_presigned_post.url, 'https://mybucket.s3.amazonaws.com')
        self.assertEquals(r_generate_presigned_post.fields['acl'], 'public-read')
        self.assertEquals(r_generate_presigned_post.fields['key'], 'mykey')
        self.assertEquals(r_generate_presigned_post.fields['signature'], 'mysignature')
        self.assertEquals(r_generate_presigned_post.fields['policy'], 'mybase64 encoded policy')

    @mock.patch.object(s3_api, 's3_session_client')
    def test_upload_object_error(self, mock_client):
        mock_client.generate_presigned_post.side_effect = ClientError(
            {
                'Error': {
                    'Code': 'NotAuthorizedException',
                    'Message': 'Details/context around the exception or error'
                },
                'ResponseMetadata': {
                    'RequestId': '1234567890ABCDEF',
                    'HostId': 'host ID data will appear here as a hash',
                    'HTTPStatusCode': 400,
                    'HTTPHeaders': {'header metadata key/values will appear here'},
                    'RetryAttempts': 0
                }
            },
            'generate_presigned_post'
        )

        with self.assertRaises(S3Error):
            s3_api.generate_presigned_post(
                mock_client,
                **self.generate_presigned_post,
            )



import boto3
import jwt
from botocore.exceptions import ClientError
from djstarter import utils as core_utils

from djaws import decorators as aws_decorators
from djaws.cognito.exceptions import (CognitoError, CognitoUsernameExistsError, CognitoUserNotFoundError,
                                      CognitoUserPasswordValidationError, CognitoUserNotAuthorizedError,
                                      CognitoUserNotConfirmedError)
from djaws.utils import ClientErrorResponse


def cognito_session_client(*args, **kwargs):
    session = boto3.session.Session()
    return session.client('cognito-idp', *args, **kwargs,)


@aws_decorators.wrap_and_reraise_cognito_errors
def signup_new_user(cognito_client, client_id, name, username, password, phone_number):
    kwargs = {
        'ClientId': client_id,
        'Username': username,
        'Password': password,
        'UserAttributes': [
            {
                'Name': "name",
                'Value': name
            },
            {
                'Name': "phone_number",
                'Value': phone_number
            }
        ],
    }
    try:
        r_signup = cognito_client.sign_up(**kwargs)
    except ClientError as e:
        raise CognitoClientErrorResponse(e.response).to_cognito_exception
    else:
        return SignupNewUserResponse(r_signup)


class SignupNewUserResponse(core_utils.SerializerMixin):

    class CodeDeliveryDetails:
        def __init__(self, data):
            self.attribute_name = data.get('AttributeName')
            self.delivery_medium = data.get('DeliveryMedium')
            self.destination = data.get('Destination')

    def __init__(self, data):
        self.code_deliver_details = self.CodeDeliveryDetails(data.get('CodeDeliveryDetails', dict()))
        self.user_confirmed = data.get('UserConfirmed')
        self.user_sub = data.get('UserSub')


@aws_decorators.wrap_and_reraise_cognito_errors
def admin_confirm_sign_up(cognito_client, user_pool_id, username):
    try:
        r_confirm = cognito_client.admin_confirm_sign_up(
            UserPoolId=user_pool_id,
            Username=username
        )
    except ClientError as e:
        raise CognitoClientErrorResponse(e.response).to_cognito_exception
    else:
        return AdminConfirmNewUserResponse(r_confirm)


class AdminConfirmNewUserResponse(core_utils.SerializerMixin):
    class ResponseMetadata:
        class HTTPHeaders:
            def __init__(self, data):
                self.date = data.get('date')
                self.content_type = data.get('content-type')
                self.content_length = data.get('content-length')
                self.connection = data.get('connection')
                self.x_amzn_requestid = data.get('x-amzn-requestid')

        def __init__(self, data):
            self.request_id = data.get('RequestId')
            self.http_status_code = data.get('HTTPStatusCode')
            self.http_headers = self.HTTPHeaders(data.get('HTTPHeaders', dict()))
            self.retry_attempts = data.get('RetryAttempts')

    def __init__(self, data):
        self.response_metadata = self.ResponseMetadata(data.get('ResponseMetadata', dict()))


@aws_decorators.wrap_and_reraise_cognito_errors
def login(cognito_client, user_pool_client_id, username, password):
    try:
        r_login = cognito_client.initiate_auth(
            AuthFlow='USER_PASSWORD_AUTH',
            AuthParameters={
                'USERNAME': username,
                'PASSWORD': password,
            },
            ClientId=user_pool_client_id
        )
    except ClientError as e:
        raise CognitoClientErrorResponse(e.response).to_cognito_exception
    else:
        return InitAuthResponse(r_login)


@aws_decorators.wrap_and_reraise_cognito_errors
def admin_login(cognito_client, user_pool_client_id, user_pool_id, username, password):
    try:
        r_login = cognito_client.admin_initiate_auth(
            AuthFlow='ADMIN_USER_PASSWORD_AUTH',
            AuthParameters={
                'USERNAME': username,
                'PASSWORD': password
            },
            ClientId=user_pool_client_id,
            UserPoolId=user_pool_id,
        )
    except ClientError as e:
        raise CognitoClientErrorResponse(e.response).to_cognito_exception
    return InitAuthResponse(r_login)


@aws_decorators.wrap_and_reraise_cognito_errors
def admin_refresh_access_token(cognito_client, user_pool_client_id, user_pool_id, refresh_token):
    """
    :param cognito_client:
    :param user_pool_client_id:
    :param user_pool_id:
    :param refresh_token:
    :return:
    :exception MarzUserNotAuthorizedError
    """
    try:
        r_refresh = cognito_client.admin_initiate_auth(
            AuthFlow='REFRESH_TOKEN_AUTH',
            AuthParameters={
                'REFRESH_TOKEN': refresh_token,
            },
            ClientId=user_pool_client_id,
            UserPoolId=user_pool_id,
        )
    except ClientError as e:
        raise CognitoClientErrorResponse(e.response).to_cognito_exception
    return InitAuthResponse(r_refresh)


class InitAuthResponse(core_utils.SerializerMixin):
    class AuthenticationResult:
        def __init__(self, data):
            self.access_token = data.get('AccessToken')
            self.expires_in = data.get('ExpiresIn')
            self.id_token = data.get('IdToken')
            self.refresh_token = data.get('RefreshToken')
            self.token_type = data.get('TokenType')

    def __init__(self, data):
        self.authentication_result = self.AuthenticationResult(data.get('AuthenticationResult', dict()))
        self.challenge_parameters = data.get('ChallengeParameters')

    @property
    def user_id(self):
        return jwt.decode(self.authentication_result.access_token, options={"verify_signature": False})['sub']

    @property
    def is_admin_user(self):
        scope = jwt.decode(self.authentication_result.access_token, options={"verify_signature": False})['scope']
        return scope == 'aws.cognito.signin.user.admin'

    @property
    def access_token(self):
        return self.authentication_result.access_token

    @property
    def refresh_token(self):
        return self.authentication_result.refresh_token


class CognitoClientErrorResponse(ClientErrorResponse):
    EXC_MAP = {
        'InternalErrorException': CognitoError,
        'InvalidPasswordException': CognitoUserPasswordValidationError,
        'NotAuthorizedException': CognitoUserNotAuthorizedError,
        'UsernameExistsException': CognitoUsernameExistsError,
        'UserNotConfirmedException': CognitoUserNotConfirmedError,
        'UserNotFoundException': CognitoUserNotFoundError,
    }

    @property
    def to_cognito_exception(self):
        return self.EXC_MAP.get(self.exception_class, CognitoError)(self.__dict__)



import boto3
from botocore.exceptions import ClientError
from django.conf import settings

from djaws import decorators as aws_decorators
from djaws.cloudformation.exceptions import CFError
from djaws.utils import ClientErrorResponse

from djstarter import utils as dj_utils


def cf_session_client(*args, **kwargs):
    session = boto3.session.Session()
    return session.client(
        'cloudformation',
        region_name=settings.AWS_DEFAULT_REGION,
        aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
        aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY,
        *args,
        **kwargs
    )


@aws_decorators.wrap_and_reraise_cloudformation_errors
def describe_stacks(cf_client, stack_name):
    try:
        r_stacks = cf_client.describe_stacks(StackName=stack_name)
    except ClientError as e:
        raise CFClientErrorResponse(e.response).to_cf_exception
    else:
        return DescribeStacksResponse(r_stacks)


class DescribeStacksResponse(dj_utils.SerializerMixin):
    class Stack:
        def __init__(self, data):
            self.stack_id = data.get('StackId')
            self.stack_name = data.get('StackName')
            self.description = data.get('Description')
            self.creation_time = data.get('CreationTime')
            self.stack_status = data.get('StackStatus')

    def __init__(self, data):
        self.stacks = [self.Stack(s) for s in data.get('Stacks', list())]

    @property
    def first_stack(self):
        return self.stacks[0]


class CFClientErrorResponse(ClientErrorResponse):
    EXC_MAP = dict()

    @property
    def to_cf_exception(self):
        return self.EXC_MAP.get(self.exception_class, CFError)(self.__dict__)



from djstarter import utils as core_utils


class ClientErrorResponse(core_utils.SerializerMixin):
    class Error:
        def __init__(self, data):
            self.code = data['Code']
            self.message = data['Message']

    class ResponseMetadata:
        def __init__(self, data):
            self.host_id = data.get('HostId')
            self.http_headers = data['HTTPHeaders']
            self.http_status_code = data['HTTPStatusCode']
            self.request_id = data['RequestId']
            self.retry_attempts = data['RetryAttempts']

    def __init__(self, data):
        self.error = self.Error(data['Error'])
        self.message = data.get('message')
        self.response_metadata = self.ResponseMetadata(data['ResponseMetadata'])

    @property
    def exception_class(self):
        return self.error.code



from djaws.exceptions import AWSError


class S3Error(AWSError):
    """All S3 Exceptions in the AWS App inherit from this class"""


class BucketAlreadyExists(S3Error):
    """Bucket Already Exists Error"""


class BucketAlreadyOwnedByYouError(S3Error):
    """Bucket Already Owned By You"""



from django.core.management.base import BaseCommand

from djaws import s3_api


class Command(BaseCommand):
    help = 'Create Public S3 Bucket'

    def add_arguments(self, parser):
        parser.add_argument(
            "--bucket_name",
            required=True,
            type=str,
            help="S3 Bucket Name",
        )
        parser.add_argument(
            "--region",
            default='us-east-1',
            type=str,
            help="S3 Region",
        )

    def handle(self, *args, **kwargs):

        bucket = kwargs['bucket_name']
        region = kwargs['region']
        self.stdout.write(self.style.MIGRATE_LABEL(f'Creating Public S3 Bucket: {bucket}'))
        s3_client = s3_api.s3_session_client()
        try:
            s3_api.create_public_access_bucket(s3_client, bucket=bucket, region=region)
        except Exception as e:
            self.stdout.write(self.style.ERROR(f'Error while running command:\n{str(e)}'))
            raise e
        else:
            self.stdout.write(self.style.MIGRATE_LABEL(f'Successfully created Public S3 Bucket: {bucket}'))




import json
from importlib.resources import open_text
from unittest import mock

from botocore.exceptions import ClientError
from django.test import TestCase

from djaws import cognito_api
from djaws.cognito.exceptions import (CognitoUsernameExistsError, CognitoUserPasswordValidationError,
                                    CognitoUserNotAuthorizedError)


class SignupNewUserTests(TestCase):
    """
    Signup New User Tests
    """

    @classmethod
    def setUpTestData(cls):
        with open_text('djaws.tests.schemas.cognito', 'signup_new_user.json') as signup_json:
            cls.r_signup_data = json.loads(signup_json.read())
        cls.signup_new_user = {
            'client_id': 'abcdefghijk',
            'name': 'John Doe',
            'username': 'user123',
            'password': 'password',
            'phone_number': '+15551234567'
        }

    @mock.patch.object(cognito_api, 'cognito_session_client')
    def test_signup_new_user_200(self, mock_client):
        mock_client.sign_up.return_value = self.r_signup_data

        r_signup = cognito_api.signup_new_user(
            mock_client,
            **self.signup_new_user
        )

        self.assertEquals(r_signup.as_json, cognito_api.SignupNewUserResponse(self.r_signup_data).as_json)

        self.assertEquals(r_signup.code_deliver_details.attribute_name, 'phone_number')
        self.assertEquals(r_signup.code_deliver_details.destination, '+*******7639')
        self.assertEquals(r_signup.code_deliver_details.delivery_medium, 'SMS')

        self.assertEquals(r_signup.user_confirmed, False)

        self.assertEquals(r_signup.user_sub, '215ee8ac-d572-4b23-9327-78896f078f0e')

    @mock.patch.object(cognito_api, 'cognito_session_client')
    def test_signup_invalid_password_error(self, mock_client):
        mock_client.sign_up.side_effect = ClientError(
            {
                'Error': {
                    'Code': 'InvalidPasswordException',
                    'Message': 'Details/context around the exception or error'
                },
                'ResponseMetadata': {
                    'RequestId': '1234567890ABCDEF',
                    'HostId': 'host ID data will appear here as a hash',
                    'HTTPStatusCode': 400,
                    'HTTPHeaders': {'header metadata key/values will appear here'},
                    'RetryAttempts': 0
                }
            },
            'get_parameter'
        )

        with self.assertRaises(CognitoUserPasswordValidationError):
            cognito_api.signup_new_user(
                mock_client,
                **self.signup_new_user
            )

    @mock.patch.object(cognito_api, 'cognito_session_client')
    def test_signup_not_authorized_error(self, mock_client):
        mock_client.sign_up.side_effect = ClientError(
            {
                'Error': {
                    'Code': 'NotAuthorizedException',
                    'Message': 'Details/context around the exception or error'
                },
                'ResponseMetadata': {
                    'RequestId': '1234567890ABCDEF',
                    'HostId': 'host ID data will appear here as a hash',
                    'HTTPStatusCode': 400,
                    'HTTPHeaders': {'header metadata key/values will appear here'},
                    'RetryAttempts': 0
                }
            },
            'get_parameter'
        )

        with self.assertRaises(CognitoUserNotAuthorizedError):
            cognito_api.signup_new_user(
                mock_client,
                **self.signup_new_user
            )

    @mock.patch.object(cognito_api, 'cognito_session_client')
    def test_signup_username_exists_error(self, mock_client):
        mock_client.sign_up.side_effect = ClientError(
            {
                'Error': {
                    'Code': 'UsernameExistsException',
                    'Message': 'Details/context around the exception or error'
                },
                'ResponseMetadata': {
                    'RequestId': '1234567890ABCDEF',
                    'HostId': 'host ID data will appear here as a hash',
                    'HTTPStatusCode': 400,
                    'HTTPHeaders': {'header metadata key/values will appear here'},
                    'RetryAttempts': 0
                }
            },
            'get_parameter'
        )

        with self.assertRaises(CognitoUsernameExistsError):
            cognito_api.signup_new_user(
                mock_client,
                **self.signup_new_user
            )


class AdminConfirmNewUserTests(TestCase):
    """
    Admin Confirm New User Tests
    """

    @classmethod
    def setUpTestData(cls):
        with open_text('djaws.tests.schemas.cognito', 'admin_confirm_sign_up.json') as admin_confirm_json:
            cls.r_confirm_data = json.loads(admin_confirm_json.read())
        cls.admin_confirm_sign_up = {
            'username': 'user123',
            'user_pool_id': 'test-pool-id-1'
        }

    @mock.patch.object(cognito_api, 'cognito_session_client')
    def test_ok(self, mock_client):
        mock_client.admin_confirm_sign_up.return_value = self.r_confirm_data

        r_confirm = cognito_api.admin_confirm_sign_up(
            mock_client,
            **self.admin_confirm_sign_up
        )

        self.assertEquals(r_confirm.as_json, cognito_api.AdminConfirmNewUserResponse(self.r_confirm_data).as_json)

        self.assertEquals(r_confirm.response_metadata.request_id, '8551f29a-c72b-44ab-abc8-014247bd0f09')
        self.assertEquals(r_confirm.response_metadata.http_status_code, 200)

        self.assertEquals(r_confirm.response_metadata.http_headers.date, 'Sat, 16 Oct 2021 04:49:22 GMT')
        self.assertEquals(r_confirm.response_metadata.http_headers.content_type, 'application/x-amz-json-1.1')
        self.assertEquals(r_confirm.response_metadata.http_headers.content_length, '2')
        self.assertEquals(r_confirm.response_metadata.http_headers.connection, 'keep-alive')
        self.assertEquals(
            r_confirm.response_metadata.http_headers.x_amzn_requestid,
            '8551f29a-c72b-44ab-abc8-014247bd0f09'
        )

        self.assertEquals(r_confirm.response_metadata.retry_attempts, 0)

    @mock.patch.object(cognito_api, 'cognito_session_client')
    def test_admin_confirm_sign_up_invalid_password_error(self, mock_client):
        mock_client.admin_confirm_sign_up.side_effect = ClientError(
            {
                'Error': {
                    'Code': 'InvalidPasswordException',
                    'Message': 'Details/context around the exception or error'
                },
                'ResponseMetadata': {
                    'RequestId': '1234567890ABCDEF',
                    'HostId': 'host ID data will appear here as a hash',
                    'HTTPStatusCode': 400,
                    'HTTPHeaders': {'header metadata key/values will appear here'},
                    'RetryAttempts': 0
                }
            },
            'get_parameter'
        )

        with self.assertRaises(CognitoUserPasswordValidationError):
            cognito_api.admin_confirm_sign_up(
                mock_client,
                **self.admin_confirm_sign_up
            )


class LoginTests(TestCase):
    """
    Login Tests
    """

    @classmethod
    def setUpTestData(cls):
        with open_text('djaws.tests.schemas.cognito', 'initiate_auth.json') as initiate_auth_json:
            cls.r_login_data = json.loads(initiate_auth_json.read())
        cls.login = {
            'user_pool_client_id': 'test-pool-client-id-1',
            'username': 'user123',
            'password': 'password',
        }

    @mock.patch.object(cognito_api, 'cognito_session_client')
    def test_login_200(self, mock_client):
        mock_client.initiate_auth.return_value = self.r_login_data

        r_login = cognito_api.login(
            mock_client,
            **self.login
        )

        self.assertEquals(r_login.as_json, cognito_api.InitAuthResponse(self.r_login_data).as_json)
        self.assertEquals(r_login.user_id, 'ef6b26e0-7868-4941-818a-816eb9ad8c6c')
        self.assertTrue(r_login.is_admin_user)

    @mock.patch.object(cognito_api, 'cognito_session_client')
    def test_login_not_authorized_error(self, mock_client):
        mock_client.initiate_auth.side_effect = ClientError(
            {
                'Error': {
                    'Code': 'NotAuthorizedException',
                    'Message': 'Details/context around the exception or error'
                },
                'ResponseMetadata': {
                    'RequestId': '1234567890ABCDEF',
                    'HostId': 'host ID data will appear here as a hash',
                    'HTTPStatusCode': 400,
                    'HTTPHeaders': {'header metadata key/values will appear here'},
                    'RetryAttempts': 0
                }
            },
            'get_parameter'
        )

        with self.assertRaises(CognitoUserNotAuthorizedError):
            cognito_api.login(
                mock_client,
                **self.login
            )


class AdminLoginTests(TestCase):
    """
    Admin Login Tests
    """

    @classmethod
    def setUpTestData(cls):
        with open_text('djaws.tests.schemas.cognito', 'initiate_auth.json') as initiate_auth_json:
            cls.r_admin_login_data = json.loads(initiate_auth_json.read())
        cls.admin_login = {
            'user_pool_client_id': 'test-pool-client-id-1',
            'user_pool_id': 'test-pool-id-1',
            'username': 'user123',
            'password': 'password',
        }

    @mock.patch.object(cognito_api, 'cognito_session_client')
    def test_admin_login_200(self, mock_client):
        mock_client.admin_initiate_auth.return_value = self.r_admin_login_data

        r_login = cognito_api.admin_login(
            mock_client,
            **self.admin_login
        )

        self.assertEquals(r_login.as_json, cognito_api.InitAuthResponse(self.r_admin_login_data).as_json)
        self.assertEquals(r_login.user_id, 'ef6b26e0-7868-4941-818a-816eb9ad8c6c')
        self.assertTrue(r_login.is_admin_user)

    @mock.patch.object(cognito_api, 'cognito_session_client')
    def test_CognitoUserNotAuthorizedError(self, mock_client):
        mock_client.admin_initiate_auth.side_effect = ClientError(
            {
                'Error': {
                    'Code': 'NotAuthorizedException',
                    'Message': 'Details/context around the exception or error'
                },
                'ResponseMetadata': {
                    'RequestId': '1234567890ABCDEF',
                    'HostId': 'host ID data will appear here as a hash',
                    'HTTPStatusCode': 400,
                    'HTTPHeaders': {'header metadata key/values will appear here'},
                    'RetryAttempts': 0
                }
            },
            'get_parameter'
        )

        with self.assertRaises(CognitoUserNotAuthorizedError):
            cognito_api.admin_login(
                mock_client,
                **self.admin_login
            )


class AdminRefreshAccessToken(TestCase):
    """
    Admin Refresh Access Token Tests
    """

    @classmethod
    def setUpTestData(cls):
        with open_text('djaws.tests.schemas.cognito', 'initiate_auth.json') as initiate_auth_json:
            cls.r_admin_refresh_access_token_data = json.loads(initiate_auth_json.read())
        cls.admin_refresh_access_token = {
            'user_pool_client_id': 'test-pool-client-id-1',
            'user_pool_id': 'test-pool-id-1',
            'refresh_token': 'refresh-token-1',
        }

    @mock.patch.object(cognito_api, 'cognito_session_client')
    def test_ok(self, mock_client):
        mock_client.admin_initiate_auth.return_value = self.r_admin_refresh_access_token_data

        r_admin_refresh_access_token = cognito_api.admin_refresh_access_token(
            mock_client,
            **self.admin_refresh_access_token
        )

        self.assertEquals(
            r_admin_refresh_access_token.as_json,
            cognito_api.InitAuthResponse(self.r_admin_refresh_access_token_data).as_json
        )
        self.assertEquals(r_admin_refresh_access_token.user_id, 'ef6b26e0-7868-4941-818a-816eb9ad8c6c')
        self.assertTrue(r_admin_refresh_access_token.is_admin_user)

    @mock.patch.object(cognito_api, 'cognito_session_client')
    def test_CognitoUserNotAuthorizedError(self, mock_client):
        mock_client.admin_initiate_auth.side_effect = ClientError(
            {
                'Error': {
                    'Code': 'NotAuthorizedException',
                    'Message': 'Details/context around the exception or error'
                },
                'ResponseMetadata': {
                    'RequestId': '1234567890ABCDEF',
                    'HostId': 'host ID data will appear here as a hash',
                    'HTTPStatusCode': 400,
                    'HTTPHeaders': {'header metadata key/values will appear here'},
                    'RetryAttempts': 0
                }
            },
            'admin_initiate_auth'
        )

        with self.assertRaises(CognitoUserNotAuthorizedError):
            cognito_api.admin_refresh_access_token(
                mock_client,
                **self.admin_refresh_access_token
            )



from unittest.mock import Mock

from django.test import TestCase

from djaws import decorators
from djaws.cognito.exceptions import CognitoError, CognitoUserNotFoundError
from djaws.s3.exceptions import S3Error, BucketAlreadyExists
from djaws.cloudformation.exceptions import CFError


class WrapAndReraiseS3ErrorsTests(TestCase):
    def test_s3_pass_thru(self):
        func = Mock(side_effect=BucketAlreadyExists)
        decorated_function = decorators.wrap_and_reraise_s3_errors(func)
        with self.assertRaises(BucketAlreadyExists):
            decorated_function()
        self.assertEquals(func.call_count, 1)

    def test_exception(self):
        func = Mock(side_effect=Exception)
        decorated_function = decorators.wrap_and_reraise_s3_errors(func)
        with self.assertRaises(S3Error):
            decorated_function()
        self.assertEquals(func.call_count, 1)


class WrapAndReraiseCognitoErrorsTests(TestCase):
    def test_cognito_pass_thru(self):
        func = Mock(side_effect=CognitoUserNotFoundError)
        decorated_function = decorators.wrap_and_reraise_cognito_errors(func)
        with self.assertRaises(CognitoUserNotFoundError):
            decorated_function()
        self.assertEquals(func.call_count, 1)

    def test_exception(self):
        func = Mock(side_effect=Exception)
        decorated_function = decorators.wrap_and_reraise_cognito_errors(func)
        with self.assertRaises(CognitoError):
            decorated_function()
        self.assertEquals(func.call_count, 1)


class WrapAndReraiseCloudFormationErrorsTests(TestCase):
    def test_cognito_pass_thru(self):
        func = Mock(side_effect=CFError)
        decorated_function = decorators.wrap_and_reraise_cloudformation_errors(func)
        with self.assertRaises(CFError):
            decorated_function()
        self.assertEquals(func.call_count, 1)

    def test_exception(self):
        func = Mock(side_effect=Exception)
        decorated_function = decorators.wrap_and_reraise_cloudformation_errors(func)
        with self.assertRaises(CFError):
            decorated_function()
        self.assertEquals(func.call_count, 1)



from functools import wraps

from .cloudformation.exceptions import CFError
from .cognito.exceptions import CognitoError
from .s3.exceptions import S3Error


def wrap_and_reraise_s3_errors(func):

    @wraps(func)
    def wrapper_func(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except S3Error:
            raise   # Pass through S3 Exceptions
        except Exception as e:
            raise S3Error from e

    return wrapper_func


def wrap_and_reraise_cognito_errors(func):

    @wraps(func)
    def wrapper_func(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except CognitoError:
            raise   # Pass through Cognito Exceptions
        except Exception as e:
            raise CognitoError from e

    return wrapper_func


def wrap_and_reraise_cloudformation_errors(func):

    @wraps(func)
    def wrapper_func(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except CFError:
            raise   # Pass through CFError Exceptions
        except Exception as e:
            raise CFError from e

    return wrapper_func



import logging
import os
import uuid

import boto3
from botocore.exceptions import ClientError
from django.conf import settings
from djstarter import decorators, utils as core_utils

from djaws import decorators as aws_decorators
from djaws.s3.exceptions import S3Error, BucketAlreadyExists, BucketAlreadyOwnedByYouError
from djaws.utils import ClientErrorResponse

logger = logging.getLogger(__name__)


def s3_session_client(*args, **kwargs):
    session = boto3.session.Session()
    return session.client(
        's3',
        region_name=settings.AWS_DEFAULT_REGION,
        aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
        aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY,
        *args,
        **kwargs
    )


def create_public_access_bucket(s3_client, bucket, region):
    create_bucket(s3_client, bucket=bucket, region=region)
    put_public_access_block(
        s3_client,
        bucket=bucket,
        config=PublicAccessBlockConfiguration(
            block_public_acls=False,
            ignore_public_acls=False,
            block_public_policy=False,
            restrict_public_buckets=False
        )
    )
    put_bucket_cors(
        s3_client,
        bucket=bucket,
        cors_config={
            'CORSRules': [
                {
                    "AllowedHeaders": [
                        "*"
                    ],
                    "AllowedMethods": [
                        "GET"
                    ],
                    "AllowedOrigins": [
                        "*"
                    ],
                    "MaxAgeSeconds": 3000
                },
            ],
        },
    )
    return True


@aws_decorators.wrap_and_reraise_s3_errors
def create_bucket(s3_client, bucket, region, **kwargs):
    """Create an S3 bucket in a specified region

    :param s3_client: S3 Client
    :param bucket: Bucket to create
    :param region: String region to create bucket in, e.g., 'us-east-1'
    :return: True if bucket created, else False
    """

    # Create bucket
    try:
        # This is really stupid S3 quirk. Technically, us-east-1 one has no S3,
        # it's actually "US Standard", or something.
        # More here: https://github.com/boto/boto3/issues/125
        if region == "us-east-1":
            r_bucket = s3_client.create_bucket(Bucket=bucket)
        else:
            r_bucket = s3_client.create_bucket(
                Bucket=bucket,
                CreateBucketConfiguration={'LocationConstraint': region},
                **kwargs
            )
    except ClientError as e:
        raise S3ClientErrorResponse(e.response).to_s3_exception
    return CreateBucketResponse(r_bucket)


class CreateBucketResponse:
    def __init__(self, data):
        self.location = data['Location']
        # self.response_metadata = data['ResponseMetadata']


@aws_decorators.wrap_and_reraise_s3_errors
def put_bucket_cors(s3_client, bucket, cors_config):
    try:
        s3_client.put_bucket_cors(
            Bucket=bucket,
            CORSConfiguration=cors_config
        )
    except ClientError as e:
        raise S3ClientErrorResponse(e.response).to_s3_exception
    else:
        return True


@aws_decorators.wrap_and_reraise_s3_errors
def put_public_access_block(s3_client, bucket, config):
    try:
        s3_client.put_public_access_block(
            Bucket=bucket,
            PublicAccessBlockConfiguration={
                'BlockPublicAcls': config.block_public_acls,
                'IgnorePublicAcls': config.ignore_public_acls,
                'BlockPublicPolicy': config.block_public_policy,
                'RestrictPublicBuckets': config.restrict_public_buckets
            },
        )
    except ClientError as e:
        raise S3ClientErrorResponse(e.response).to_s3_exception
    else:
        return True


class PublicAccessBlockConfiguration:
    def __init__(self, block_public_acls, ignore_public_acls, block_public_policy, restrict_public_buckets):
        self.block_public_acls = block_public_acls
        self.ignore_public_acls = ignore_public_acls
        self.block_public_policy = block_public_policy
        self.restrict_public_buckets = restrict_public_buckets


@decorators.timing
@aws_decorators.wrap_and_reraise_s3_errors
def upload_object(s3_client, data, bucket, uri, key=None):
    """Upload an object to an S3 bucket

    :param key:
    :param s3_client:
    :param data: Data to upload
    :param bucket: s3 Bucket to upload to
    :param uri: Uri of object
    :return: Public S3 Url
    """

    extra_args = {
        'ContentType': core_utils.get_mimetype(uri),
        'ACL': 'public-read',
    }

    # If S3 object_name was not specified, use uuid file_name
    if key is None:
        key = f'{uuid.uuid4()}{core_utils.get_file_ext(uri)}'

    try:
        s3_client.put_object(
            Body=data,
            Bucket=bucket,
            Key=key,
            **extra_args
        )
    except ClientError as e:
        raise S3ClientErrorResponse(e.response).to_s3_exception

    return UploadFileResponse(object_name=key, bucket=bucket)


class UploadFileResponse:
    def __init__(self, object_name, bucket):
        self.object_name = object_name
        self.bucket = bucket

    @property
    def public_s3_url(self):
        return get_public_s3_url(bucket=self.bucket, key=os.path.basename(self.object_name))


@aws_decorators.wrap_and_reraise_s3_errors
def generate_presigned_post(s3_client, bucket, object_name, fields=None, conditions=None, expiration=3600):
    """Generate a presigned URL S3 POST request to upload a file

    :param s3_client:K
    :param bucket: string
    :param object_name: string
    :param fields: Dictionary of prefilled form fields
    :param conditions: List of conditions to include in the policy
    :param expiration: Time in seconds for the presigned URL to remain valid
    :return: Dictionary with the following keys:
        url: URL to post to
        fields: Dictionary of form fields and values to submit with the POST
    :return: None if error.
    """
    try:
        r_presigned_url = s3_client.generate_presigned_post(
            bucket,
            object_name,
            Fields=fields,
            Conditions=conditions,
            ExpiresIn=expiration
        )
    except ClientError as e:
        raise S3ClientErrorResponse(e.response).to_s3_exception
    return PresignedPostResponse(r_presigned_url)


class PresignedPostResponse:
    def __init__(self, data):
        self.url = data['url']
        self.fields = data['fields']


class S3ClientErrorResponse(ClientErrorResponse):
    EXC_MAP = {
        'BucketAlreadyExists': BucketAlreadyExists,
        'BucketAlreadyOwnedByYou': BucketAlreadyOwnedByYouError,
    }

    @property
    def to_s3_exception(self):
        return self.EXC_MAP.get(self.exception_class, S3Error)(self.__dict__)


def get_public_s3_url(bucket, key):
    return f'https://{bucket}.s3.amazonaws.com/{key}'



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['djaws',
 'djaws.cloudformation',
 'djaws.cognito',
 'djaws.management',
 'djaws.management.commands',
 'djaws.s3',
 'djaws.tests',
 'djaws.tests.schemas',
 'djaws.tests.schemas.cloudformation',
 'djaws.tests.schemas.cognito',
 'djaws.tests.schemas.s3']

package_data = \
{'': ['*']}

install_requires = \
['Django>=3.2.4,<3.3.0',
 'PyJWT[crypto]>=2.3.0,<3.0.0',
 'boto3>=1.20.35,<1.21.0',
 'dj-starter>=0.1.1,<0.2.0',
 'django-environ>=0.8.1,<0.9.0',
 'psycopg2-binary>=2.9.2,<3.0.0']

setup_kwargs = {
    'name': 'dj-aws',
    'version': '0.1.3',
    'description': 'Django + AWS made easy',
    'long_description': None,
    'author': 'Adrian',
    'author_email': 'adrian@rydeas.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/adrianmeraz/dj-aws',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
